********************************************************************************
***************** Introduction to Regression Diagnostics ***********************
********************************************************************************
********************** Unusual and influential data ****************************
********************************************************************************
* # Outliers: In linear regression, an outlier is an observation with large residual. 
// In other words, it is an observation whose dependent-variable value is unusual given its values on the predictor variables. 
// An outlier may indicate a sample peculiarity or may indicate a data entry error or other problem.

* # Leverage: An observation with an extreme value on a predictor variable is called a point with high leverage. 
// Leverage is a measure of how far an observation deviates from the mean of that variable. 
// These leverage points can have an effect on the estimate of regression coefficients.

* # Influence: An observation is said to be influential if removing the observation substantially changes the estimate of coefficients. 
// Influence can be thought of as the product of leverage and outlierness.

// How can we identify these three types of observations? Let's look at an example dataset called crime. 
// This dataset  appears in Statistical Methods for Social Sciences, Third Edition by Alan Agresti and Barbara Finlay (Prentice Hall, 1997). 
// The variables are state id (sid), state name (state), violent crimes per 100,000 people (crime), murders per 1,000,000 (murder), 
// the percent of the population living in metropolitan areas (pctmetro), the percent of the population that is white (pctwhite), 
// percent of population with a high school education or above (pcths), percent of population living under poverty line (poverty), 
// and percent of population that are single parents (single). 

use http://www.ats.ucla.edu/stat/stata/webbooks/reg/crime.dta, clear
describe

summarize crime murder pctmetro pctwhite pcths poverty single

* Let's say that we want to predict crime by pctmetro, poverty, and single. 
// That is to say, we want to build a linear regression model between the response variable crime and the independent variables pctmetro, poverty and single. 
// We will first look at the scatter plots of crime against each of the predictor variables before the regression analysis 
// so we will have some ideas about potential problems. 
// We can create a scatterplot matrix of these variables as shown below.

graph matrix crime pctmetro poverty single

// The graphs of crime with other variables show some potential problems.  
// In every plot, we see a data point that is far away from the rest of the data points. 
// Let's make individual graphs of crime with pctmetro and poverty and single so we can get a better view of these scatterplots.  
// We will add the mlabel(state) option to label each marker with the state name to identify outlying states.

scatter crime pctmetro, mlabel(state)
scatter crime poverty, mlabel(state)
scatter crime single, mlabel(state)

// All the scatter plots suggest that the observation for state = dc is a point that requires extra attention 
// since it stands out away from all of the other points. We will keep it in mind when we do our regression analysis.

// Let's try the regression command predicting crime from pctmetro poverty and single. 
// We will go step-by-step to identify all the potentially unusual or influential points afterwards. 

regress crime pctmetro poverty single

// Let's examine the studentized residuals as a first means for identifying outliers. 
// Below we use the predict command with the rstudent option to generate studentized residuals and we name the residuals r.   
// We can choose any name we like as long as it is a legal Stata variable name. 
// Studentized residuals are a type of standardized residual that can be used to identify outliers.

predict r, rstudent

// Let's examine the residuals with a stem and leaf plot.

stem r

// We see three residuals that stick out, -3.57, 2.62 and 3.77. 
// The stem and leaf display helps us see some potential outliers, 
// but we cannot see which state (which observations) are potential outliers.  
// Let's sort the data on the residuals and show the 10 largest and 10 smallest residuals along with the state id and state name.  
// Note that in the second list command the -10/l the last value is the letter "l", NOT the number one.

sort r
list sid state r in 1/10
list sid state r in -10/l

// We should pay attention to studentized residuals that exceed +2 or -2, and get even more concerned about residuals that exceed +2.5 or -2.5 
// and even yet more concerned about residuals that exceed +3 or -3.  These results show that DC and MS are the most worrisome observations followed by FL.

// Another way to get this kind of output is with a command called "hilo".  
// You can download hilo from within Stata by typing "findit hilo".

// Once installed, you can type the following and get output similar to that above by typing just one command.

hilo r state

// Let's show all of the variables in our regression where the studentized residual exceeds +2 or -2, 
// i.e., where the absolute value of the residual exceeds 2. 

list state r crime pctmetro poverty single if abs(r) > 2
 
// We see the data for the three potential outliers we identified, namely Florida, Mississippi and Washington D.C. 
// Looking carefully at these three observations, we couldn't find any data entry error, 
// though we may want to do another regression analysis with the extreme point such as DC deleted. 

// Now let's look at the leverage's to identify observations that will have potential great influence on regression coefficient estimates.  

predict lev, leverage
stem lev

// We use the show(5) high options on the "hilo" command to show just the 5 largest observations (the high option can be abbreviated as h).  

hilo lev state, show(5) high

// We see that DC has the largest leverage.

// Generally, a point with leverage greater than (2k+2)/n should be carefully examined. 
// Here k is the number of predictors and n is the number of observations. In our example, we can do the following. 

display (2*3+2)/51

list crime pctmetro poverty single state lev if lev >.156

// As we have seen, DC is an observation that both has a large residual and large leverage.  
// Such points are potentially the most influential.  
// We can make a plot that shows the leverage by the residual squared and look for observations that are jointly high on both of these measures.  
// We can do this using the lvr2plot command. lvr2plot stands for leverage versus residual squared plot. 
// Using residual squared instead of residual itself, the graph is restricted to the first quadrant and the relative positions of data points are preserved. 
// This is a quick way of checking potential influential observations and outliers at the same time. Both types of points are of great concern for us.  

lvr2plot, mlabel(state)

// The two reference lines are the means for leverage, horizontal, and for the normalized residual squared, vertical. 
// The points that immediately catch our attention is DC (with the largest leverage) and MS (with the largest residual squared). 
// We'll look at those observations more carefully by listing them.

list state crime pctmetro poverty single if state=="dc" | state=="ms"

// Now let's move on to overall measures of influence, specifically let's look at Cook's D and DFITS.  
// These measures both combine information on the residual and leverage. 
// Cook's D and DFITS are very similar except that they scale differently but they give us similar answers.

// The lowest value that Cook's D can assume is zero, and the higher the Cook's D is, the more influential the point. 
// The convention cut-off point is 4/n. We can list any observation above the cut-off point by doing the following. 

predict d, cooksd
list crime pctmetro poverty single state d if d>4/51

// We do see that the Cook's D for DC is by far the largest. 

// Now let's take a look at DFITS. 
// The cut-off point for DFITS is 2*sqrt(k/n). 
// DFITS can be either positive or negative, with numbers close to zero corresponding to the points with small or zero influence. 
// As we see, dfit also indicates that DC is, by far, the most influential observation.

predict dfit, dfits
list crime pctmetro poverty single state dfit if abs(dfit)>2*sqrt(3/51)

// The above measures are general measures of influence.  

// You can also consider more specific measures of influence that assess how each coefficient is changed by deleting the observation. 
// This measure is called DFBETA and is created for each of the predictors. 
// Apparently this is more computational intensive than summary statistics such as Cook's D 
// since the more predictors a model has, the more computation it may involve. 
// We can restrict our attention to only those predictors that we are most concerned with to see how well behaved those predictors are. 

// In Stata, the "dfbeta" command will produce the DFBETAs for each of the predictors. 
// The names for the new variables created are chosen by Stata automatically and begin with the letters DF. 

dfbeta

// This created three variables, _dfbeta_1, _dfbeta_2, and _dfbeta_3.  Let's look at the first 5 values. 

// If we consider _dfbeta_3 (DFBETA for predictor single), then by being included in the analysis (as compared to being excluded), 
// certain observation (state) increases the coefficient for single by (the values of _dfbeta_3) times (standard errors of the coefficient on single)

// Since the inclusion of an observation could either contribute to an increase or decrease in a regression coefficient, 
// DFBETAs can be either positive or negative.  

// A DFBETA value in excess of  2/sqrt(n) merits further investigation. 
// In this example, we would be concerned about absolute values in excess of 2/sqrt(51) or .28.

// We can plot all three DFBETA values against the state id in one graph shown below.
// We add a line at .28 and -.28 to help us see potentially troublesome observations.  

scatter _dfbeta_1 _dfbeta_2 _dfbeta_3 sid, ylabel(-1(.5)3) yline(.28 -.28)
 
// We see the largest value is about 3.0 for _dfbeta_3 (for DFBETA predictor single).

// We can repeat this graph with the mlabel() option in the graph command to label the points. 
// With the graph above we can identify which DFBETA is a problem, 
// and with the graph below we can associate that observation with the state that it originates from. 

scatter _dfbeta_1 _dfbeta_2 _dfbeta_3 sid, ylabel(-1(.5)3) yline(.28 -.28) ///
mlabel(state state state)
  
// Now let's list those observations with _dfbeta_3 larger than the cut-off value.

list _dfbeta_3 state crime pctmetro poverty single if abs(_dfbeta_3) > 2/sqrt(51)

// Below we summarizes the general rules of thumb we use for these measures to identify observations worthy of further investigation 

* # Measure		Value
* # leverage	>(2k+2)/n
* # abs(rstu)	> 2
* # Cook's D	> 4/n
* # abs(DFITS)	> 2*sqrt(k/n)
* # abs(DFBETA)	> 2/sqrt(n)

//(where k is the number of predictors and n is the number of observations).

* There are also several other graphs that can be used to search for unusual and influential observations. 

// The "avplot" command graphs an added-variable plot. 
// It is also called a partial-regression plot and is very useful in identifying influential points. 

* From Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire, Ch. 7.
* Added-variable plots are valuable diagnostic tools known by different, names including partial-regression leverage plots, 
// adjusted partial residual plots, or adjusted variable plots. 
// They depict the relationship between y and one x variable, adjusting for the effects of other x variables. 
// If we regressed y on x2 and x3, and likewise regressed x1 on x2 and x3, 
// then took the residuals from each regression and graphed these residuals in a scatterplot, 
// we would obtain an added-variable plot for the relationship between y and x1, adjusted for x2 and x3. 
// An avplot command performs the necessary calculations automatically. 

// For example, consider the avplot for single

avplot single, mlabel(state)

// The graph shows crime by single after both crime and single have been adjusted for all other predictors in the model.   
// The line plotted has the same slope as the coefficient for single.   
// This plot shows how the observation for DC influences the coefficient.  
// You can see how the regression line is tugged upwards trying to fit through the extreme value of DC.  
// Alaska and West Virginia may also exert substantial leverage on the coefficient of single. 

// Stata also has the avplots command that creates an added variable plot for all of the variables, which can be very useful when you have many variables.  
// It does produce small graphs, but these graphs can quickly reveal whether you have problematic observations based on the added variable plots.

avplots

// DC has appeared as an outlier as well as an influential point in every analysis.   
// Since DC is really not a state, we can use this to justify omitting it from the analysis saying that we really wish to just analyze states. 
// First, let's repeat our analysis including DC by just typing regress.

regress

// Now, let's run the analysis omitting DC by including if state != "dc" on the regress command 
// (here != stands for "not equal to" but you could also use ~= to mean the same thing). 

regress crime pctmetro poverty single if state!="dc"

// As we expect, deleting DC made a large change in the coefficient for single. 
// The coefficient for single dropped from 132.4 to 89.4.  
// After having deleted DC, we would repeat the process we have illustrated in this session to search for any other outlying and influential observations.

// Finally, we showed that the avplot command can be used to searching for outliers among existing variables in the model,
// but we should note that the avplot command not only works for the variables in the model, 
// it also works for variables that are not in the model, which is why it is called added-variable plot. 

// Let's use the regression that includes DC as we want to continue to see ill-behavior caused by DC as a demonstration for doing regression diagnostics. 
// We can do an avplot on variable pctwhite. 

regress crime pctmetro poverty single
avplot pctwhite

// We have "coef=-3.509". It is the coefficient for pctwhite if it were put in the model. 
// We can check that by doing a regression as below.

regress crime pctmetro pctwhite poverty single

*** Summary  

// In this section, we explored a number of methods of identifying outliers and influential points. 
// In a typical analysis, you would probably use only some of these methods. 
// Generally speaking, there are two types of methods for assessing outliers: 
* # statistics such as residuals, leverage, Cook's D and DFITS, that assess the overall impact of an observation on the regression results, 
* # and statistics such as DFBETA that assess the specific impact of an observation on the regression coefficients. 

// In our example, we found that DC was a point of major concern. 
// We performed a regression with it and without it and the regression equations were very different. 
// We can justify removing it from our analysis by reasoning that our model is to predict crime rate for states, not for metropolitan areas.

*** Potential solution (from Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press, Ch. 8)

* If the influential case can be attributed unquestionably to a measurement error, 
// you should either correct for the error or delete the observation from the file.
// If the influential observation result from extreme values of the dependent variable, it is reasonable to use median regression (produced by "qreg").

* (from Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire,  Ch. 8)
* [A median regression is quite similar to the OLS regression. 
// Whereas the sum of the squared residuals is minimized in OLS regression, the sum of the absolute residuals is minimized when applying median regression.
// Squaring residuals in OLS means that large residuals are more heavily weighted than small residuals.
// This property is lost in median regression, so it is less sensitive to outliers than OLS regression.

// Median regression takes its name from its predicted values, which are estimates of the median of the dependent variable conditional
// on the values of the independent variables.
// In OLS, the predicted values are estimates of the conditional means of the dependent variable.

// Stata treats median regression as a special case of a quantile regression.
// In quantile regression, the coefficient is estimated so that the sum of weighted absolute residuals is minimized.]

* Almost always, however, influential onservations result from an incomplete specified model.
********************************************************************************
********************************************************************************
********************* Checking Normality of Residuals **************************
********************************************************************************
/* Many researchers believe that multiple regression requires normality. This is not the case. 
Normality of residuals is only required for valid hypothesis testing, that is, the normality assumption assures that 
the p-values for the t-tests and F-test will be valid. 
Normality is not required in order to obtain unbiased estimates of the regression coefficients. 
OLS regression merely requires that the residuals (errors) be identically and independently distributed. 

After we run a regression analysis, we can use the predict command to create residuals and then use commands such as 
"kdensity", "qnorm" and "pnorm" to check the normality of the residuals.

Use the elemapi2 data file for these analyses.  
Let's predict academic performance (api00) from percent receiving free meals (meals), 
percent of English language learners (ell), and percent of teachers with emergency credentials (emer). */

use elemapi2.dta, clear

regress api00 meals ell emer

// We then use the predict command to generate residuals.

predict r, resid

// Below we use the "kdensity" command to produce a kernel density plot with the normal option requesting that a normal density be overlaid on the plot. 
// "kdensity" stands for kernel density estimate. It can be thought of as a histogram with narrow bins and moving average. 

kdensity r, normal

// The "pnorm" command graphs a standardized normal probability (P-P) plot 
// while "qnorm" plots the quantiles of a variable against the quantiles of a normal distribution. 
// "pnorm" is sensitive to non-normality in the middle range of data and 
// "qnorm" is sensitive to non-normality near the tails. 
// As you see below, the results from "pnorm" show no indications of non-normality, 
// while the "qnorm" command shows a slight deviation from normal at the upper tail, as can be seen in the kdensity above.  
// Nevertheless, this seems to be a minor and trivial deviation from normality. We can accept that the residuals are close to a normal distribution.

pnorm r

qnorm r

// There are also numerical tests for testing normality. 
// One of the tests is the test written by Lawrence C. Hamilton, Dept. of Sociology, Univ. of New Hampshire, called "iqr". 
// You can get this program from Stata by typing "findit iqr".

// "iqr" stands for inter-quartile range and assumes the symmetry of the distribution. 
// Severe outliers consist of those points that are either 3 inter-quartile-ranges below the first quartile 
// or 3 inter-quartile-ranges above the third quartile. 
// The presence of any severe outliers should be sufficient evidence to reject normality at a 5% significance level. 
// Mild outliers are common in samples of any size. In our case, we don't have any severe outliers and the distribution seems fairly symmetric. 
// The residuals have an approximately normal distribution. 

iqr r

* Definitions

* #      IQR (Interquartile Range) = 75th percentile - 25th percentile
* #      Pseudo standard deviation = IQR/1.349
* #      10% trim mean             = Average of cases between 10th and
* #                                     90th percentiles
* #      Inner fences              = Q(25)-1.5IQR and Q(75)+1.5IQR
* #      Outer fences              = Q(25)-3IQR   and Q(75)+3IQR
* #      Mild outlier              = Q(25)-3IQR <= x < Q(25)-1.5IQR  or
* #                                  Q(75)+1.5IQR < x <= Q(75)+3IQR
* #      Severe outlier            = x < Q(25)-3IQR  or  x > Q(75)+3IQR

// Another test available is the "swilk" test which performs the Shapiro-Wilk W test for normality. 
// The p-value is based on the assumption that the distribution is normal. 
// In our example, it is very large (.51), indicating that we cannot reject that r is normally distributed. 

swilk r
********************************************************************************
********************************************************************************
********************* Checking Homoscedasticity of Residuals *******************
********************************************************************************
*** Particularly, homoscedasticity (constant variance) of the errors
* # (a) versus time
* # (b) versus the predictions (or versus any independent variable)

* (from Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press)
* The assumption that Var(ei)=sigma_sqr requires that the variance of the errors be the same for all values of the independent variables.
// This assumption is called homoscedasticity, and its violation is called heteroskedasticity.
// Unlike the violation of E(ei)=0, heteroskedasticity does not lead to biased estimates.
// But when the homoskedasticity assumption is violated, the estimated coefficients of a regression model are not efficient (not BLUE).
// With inefficient estimation, there is an increasing probability 
// that a particular estimated regression coefficient deviates from the true for the population.
// That is, heteroskedasticity causes the standard errors of the coefficient to be incorrect, 
// and that obviously has an impact on any statistical inference that you perform.

// If the model is well-fitted, there should be no pattern to the residuals plotted against the fitted values. 
// If the variance of the residuals is non-constant then the residual variance is said to be "heteroscedastic." 
// There are graphical and non-graphical methods for detecting heteroscedasticity. 
// A commonly used graphical method is to plot the residuals versus fitted (predicted) values.  
// We do this by issuing the "rvfplot" command. 
// Below we use the rvfplot command with the yline(0) option to put a reference line at y=0. 

regress api00 acs_k3 grad_sch col_grad some_col

rvfplot, yline(0)

// We see that the pattern of the data points is getting a little narrower towards the right end, which is an indication of heteroscedasticity.

// Now let's look at a couple of commands that test for heteroscedasticity.

estat imtest, white

// The skew and kurtosis tests are also reported with chi-squares, 
// small chi-squares and big p-values mean that skew and kurtosis aren't a problem.

estat hettest

// The first test on heteroskedasticity given by imtest is the White's test 
// and the second one given by hettest is the Breusch-Pagan test. 
// Both test the null hypothesis that the variance of the residuals is homogenous. 
// Therefore, if the p-value is very small, we would have to reject the hypothesis 
// and accept the alternative hypothesis that the variance is not homogenous. 

// So in this case, the evidence is against the null hypothesis that the variance is homogeneous. 
// These tests are very sensitive to model assumptions, such as the assumption of normality. 

// Therefore it is a common practice to combine the tests with diagnostic plots to make a judgment on the severity of the heteroscedasticity 
// and to decide if any correction is needed for heteroscedasticity.

* Below we introduce one variation of the residual-versus-fitted plot, which emphasizes the variance of the residuals.
// One can divide the x axis into k groups with the same number of observations and then draw a box plot of the studentized residuals.

// To do this one again runs the regression model and get the predicted values, and the studentized residuals:

regress api00 acs_k3 grad_sch col_grad some_col
predict yhat
predict rstud, rstud

// For this example, we choose the number of groups used for the x axis so that each box contains roughly 50 observations
// (this grouping is called grouping by quantiles):

local groups = round(e(N)/50,1)

// We define the local macro groups. The saved result e(N) stores the number of observations of the last regression model.

// In Stata, one can group by quantiles with the command "xtile"

xtile groups = yhat, nquantiles(`groups')
* run lines where you define local and use local together, otherwise local is forgotten by Stata
graph box rstud, over(groups)

*** Potential solution (from Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press)

* Often you can simply transform the dependent variable to remove heteroskedasticity. The transformation should end in a symetric variable.
// For right-skewed variables, a logarithmic transformation is often sufficient.
// Also the "boxcox" command allows you to transform a variable so that it is as symmetric as possible. 

* If transforming the dependent variable does not remove heteroskedasticity in the regression model, 
// you cannot use the standard errors of the estimated coefficient (as they are given in the regression output) for a significance tests.

// If you are nevertheless interested in a significance test, you might want to try "vce(robust)" option in the regression command.
// When you use this option, the standard errors are computed so that homoskedasticity of the error terms need not be assumed.

*** Consider example from Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire

// T:\Labor_economics\ES\ES4\Datasets\Nations2.dta, clear
use D:\Study\CERGE-EI\TA\Labor_economics\ES\ES4\Datasets\Nations2.dta, clear
describe

* Calculate centered versions of urban and loggdp, named urban0 and loggdp0 (subtracting the mean values). 
* The interaction term urb_gdp is then defined as the product of urban0 times loggdp0.

generate loggdp = log10(gdp)
summarize urban
generate urban0 = urban - r(mean)
label variable urban0 "Percent urban, centered"
summarize loggdp
generate loggdp0 = loggdp - r(mean)
label variable loggdp0 "log10(GDP per cap), centered"
generate urb_gdp = urban0 * loggdp0
label variable urb_gdp "interaction urban0*loggdp0"
generate logco2 = log10(co2)

* Regress logco2 on the centered main effects loggdp0 and urban0, along with the interaction term urb_gdp.

regress logco2 loggdp0 urban0 urb_gdp

* To repeat this same regression but with robust standard errors, we just add the vce(robust) option:

regress logco2 loggdp0 urban0 urb_gdp, vce(robust)

* Descriptive aspects of the regression  the coefficients and R2  are identical with or without robust standard errors. 
// On the other hand the robust standard errors themselves, along with confidence intervals, t and F tests, 
// differ from their non-robust counterparts seen earlier. 
// The differences here are slight, however. 
// The basic results for this example do not depend on assuming errors that are independent and identically distributed across values of predictors.
********************************************************************************
********************************************************************************
********************* Checking Homoscedasticity of Residuals *******************
********************************************************************************
* In trying to include all important influential factors in the model, there is another risk called multicollinearity

*** Effects of Collinearity
* # Imprecise estimates of regression coefficients.
* # Slight fluctuations in correlation may lead to large differences in regression coefficients.
* # Adding or dropping cases may lead to large differences in regression coefficients.
* # Increases the standard error of coefficients, thus reducing tests of significance.

* When there is a perfect linear relationship among the predictors, the estimates for a regression model cannot be uniquely computed. 
// The term collinearity implies that two variables are near perfect linear combinations of one another. 
// When more than two variables are involved it is often called multicollinearity, although the two terms are often used interchangeably.

// The primary concern is that as the degree of multicollinearity increases, 
// the regression model estimates of the coefficients become unstable and the standard errors for the coefficients can get wildly inflated. 
// We will explore some Stata commands that help to detect multicollinearity.

// We can use the "vif" command after the regression to check for multicollinearity. 
// "vif" stands for variance inflation factor. 
// As a rule of thumb, a variable whose VIF values are greater than 10 may merit further investigation. 

// Tolerance, defined as 1/VIF, is used by many researchers to check on the degree of collinearity. 
// A tolerance value lower than 0.1 is comparable to a VIF of 10. 
// It means that the variable could be considered as a linear combination of other independent variables. 
// Let's first look at the regression we did from the last section, 
// the regression model predicting api00 from meals, ell and emer and then issue the vif command. 

regress api00 meals ell emer

vif

// The VIFs look fine here. Here is an example where the VIFs are more worrisome.

regress api00 acs_k3 avg_ed grad_sch col_grad some_col

vif

// In this example, the VIF and tolerance (1/VIF) values for avg_ed grad_sch and col_grad are worrisome.  
// All of these variables measure education of the parents and the very high VIF values indicate that these variables are possibly redundant.  
// For example, after you know grad_sch and col_grad, you probably can predict avg_ed very well.  
// In this example, multicollinearity arises because we have put in too many variables that measure the same thing, parent education. 

// Let's omit one of the parent education variables, avg_ed. 

regress api00 acs_k3 grad_sch col_grad some_col

vif
 
// Note that the VIF values in the analysis below appear much better.  
// Also, note how the standard errors are reduced for the parent education variables, grad_sch and col_grad.  
// This is because the high degree of collinearity caused the standard errors to be inflated.
// With the multicollinearity eliminated, the coefficient for grad_sch, which had been non-significant, is now significant.

*** Although these are fairly reliable methods, it is difficult to determine the exact number of variables 
// involved in near linear dependencies especially when there are several complex linear associations. 

*** Other tests for assessing collinearity include inspection of the "eigenvalues," 
// and a broader "eigensystem" analysis of the corresponding condition indexes 
// These methods are generally considered as best practices for assessing linear dependencies in model data.

* A rule of thumb is that the greater the number of eigenvalues near zero, 
// the greater the number of linear dependencies among the variables. 

* To address the question how close to zero must the eigenvalues be, researchers often analyze the spectrum of eigenvalues. 
// This measure, called the condition number, is the ratio of the largest to the smallest eigenvalue: CN = (lambda_max/lambda_min). 
// A related diagnostic, the condition index, provides another yardstick against which smallness can be measured. 
// Condition indexes (CI) are calculated as follows: 
// CN = (lambda_max/lambda_i)^2 for i=1,2.
// Condition indexes reveal the number and relative strength of the near dependencies.

*** Guidelines for Interpreting Collinearity Based on Condition Numbers and Indexes

* Condition Numbers (CN)
* # If (CN < 100)			Weak
* # If (100< CN < 1000)		Moderate to Strong
* # If (CN > 1000) 			Severe

* Condition Index (CI)
* # If ((CI < 10)			Weak
* # If ((10 < CI < 30)		Moderate to Strong
* # If (CI > 30) 			Severe

* Let's introduce another command on collinearity, which incorporates the above mentioned measures. 
// The "collin" command displays several different measures of collinearity. 
// For example, we can test for collinearity among the variables we used in the two examples above.  
// Note that the "collin" command does not need to be run in connection with a regress command, 
// unlike the "vif" command which follows a regress command. 
// Also note that only predictor (independent) variables are used with the collin command. 
// You can download "collin" from within Stata by typing "findit collin".

collin acs_k3 avg_ed grad_sch col_grad some_col, corr

// We now remove avg_ed and see the collinearity diagnostics improve considerably.

collin acs_k3 grad_sch col_grad some_col, corr

// The condition number is a commonly used index of the global instability of the regression coefficients -
// a large condition number, 10 or more, is an indication of instability.

// Since with the collin command, you only give the names of the X variables, not the Y,
// if Y has missing data, you have to make sure that the same cases are analyzed by the collin command 
// that were analyzed by the regress command.
// There are various ways of doing this. 
// By adding the optional if !missing(y) you tell Stata to only analyze those cases that were NOT missing on Y.

* collin computes several collinearity diagnostic measures including
// VIF, tolerance, eigenvalues, condition index, and R-squared.  
// It will compute the eigenvalues and condition index on either 
// the raw SSCP (sum of squares and cross products) with an intercept (default) 
// or the deviation SSCP without an intercept.
// With option "corr" eigenvalues and condition index are computed from correlation matrix without a constant.

*** Summary

** Common Indicators of Collinearity
* # VIF values are large
* # individual VIF greater than 10 should be inspected
* # average VIF greater than 6

** tolerance
* # tolerance values are small, close to zero
* # tolerance less than .1

** Other Indicators of Collinearity
* # Condition index -- large values
* # Condition number -- large values (the condition number is the largest condition index)
* # Eigenvalues -- small values, close to zero
* # Determinant of correlation matrix -- very small, close to zero
* # Diagonal of R-1 (inverse of correlation matrix) -- large values, values close to one are good 
// (option "rinv" display inverse of correlation matrix.)

********************************************************************************
********************************************************************************
***************************** Checking Linearity *******************************
********************************************************************************

* When we do linear regression, we assume that the relationship between the response variable and the predictors is linear. 
// This is the assumption of linearity. If this assumption is violated, the linear regression will try to fit a straight line to data 
// that does not follow a straight line. 
// Checking the linear assumption in the case of simple regression is straightforward, since we only have one predictor. 
// All we have to do is a scatter plot between the response variable and the predictor to see if nonlinearity is present.

use elemapi2.dta, clear
regress api00 enroll

// Below we use the scatter command to show a scatterplot predicting api00 (Academic Performance Index) from enroll and use lfit to show a linear fit, 
// and then lowess to show a lowess smoother predicting api00 from enroll.

twoway (scatter api00 enroll) (lfit api00 enroll) (lowess api00 enroll)

// We clearly see some degree of nonlinearity. 

// Checking the linearity assumption is not so straightforward in the case of multiple regression. 
// We will try to illustrate some of the techniques that you can use. 
// The most straightforward thing to do is to plot the standardized residuals against each of the predictor variables in the regression model. 
// Plots that exhibit systematic structure indicate that the form of the function can be improved in some way.
// (If there is a clear nonlinear pattern, there is a problem of nonlinearity.)
// Otherwise, we should see for each of the plots just a random scatter of points. 
// Let's continue to use dataset elemapi2 here. Let's use a different model. 

regress api00 meals some_col

predict r, resid

scatter r meals

scatter r some_col

// The two residual versus predictor variable plots above do not indicate strongly a clear departure from linearity. 

// Another command for detecting non-linearity is "acprplot". 

// A component-plus residual plot (produced by "cprplot") for variable x1 graphs each residual plus its component predicted from x1:
// ei + b1*x1i
// against values of x1. 
// Such plots can be used to identify nonlinearities in the data. 
// An augmented component-plus-residual plot, a.k.a. augmented partial residual plot, (produced by "acprplot") (Mallows 1986) works somewhat better, 
// although both types often seem inconclusive. 

// [based on Hamilton L. C. (2013). Statistics with STATA: Version 12, 8th Edition. Cengage. University of New Hampshire, Ch. 7]

// Let's use the acprplot command for meals and some_col and use the lowess lsopts(bwidth(1)) options to request lowess smoothing with a bandwidth of 1.

acprplot meals, lowess lsopts(bwidth(1))

// The straight line in the Figure corresponds to the regression model. 
// The curved line reflects lowess smoothing, which would show us if there was much nonlinearity. 

// If more central parts of the lowess curve showed a systematically curved pattern, departing from the linear regression model, 
// we would have reason to doubt the models adequacy. 

// In the plot the smoothed line is very close to the ordinary regression line, and the entire pattern seems pretty uniform.
// This plot reinforces the conclusion that the present regression model adequately accounts for all nonlinearity visible in the raw data, 
// leaving none in its residuals.

acprplot some_col, lowess lsopts(bwidth(1))

// The second plot does seem more problematic at the right end. This may come from some potential influential points. 
// Overall, they don't look too bad and we shouldn't be too concerned about non-linearities in the data.

// We have seen how to use acprplot to detect nonlinearity. However our last example didn't show much nonlinearity. 
// Let's look at a more interesting example. This example is taken from "Statistics with Stata 5" by Lawrence C. Hamilton (1997, Duxbery Press). 
// The dataset we will use is called nations.dta.

use nations.dta, clear
describe

// Let's build a model that predicts birth rate (birth), from per capita gross national product (gnpcap), and urban population (urban). 
// If this were a complete regression analysis, we would start with examining the variables, but for the purpose of illustrating nonlinearity, 
// we will jump directly to the regression.

regress birth gnpcap urban

// Now, let's do the acprplot on our predictors.

acprplot gnpcap, lowess

acprplot urban, lowess

// The acprplot plot for gnpcap shows clear deviation from linearity and the one for urban does not show nearly as much deviation from linearity. 
// Now, let's look at these variables more closely.

graph matrix birth gnpcap urban, half

// We see that the relation between birth rate and per capita gross national product is clearly nonlinear 
// and the relation between birth rate and urban population is not too far off from being linear. 
// So let's focus on variable gnpcap. 
// First let's look at the distribution of gnpcap. We suspect that gnpcap may be very skewed. This may affect the appearance of the acprplot.

kdensity gnpcap, normal

// Indeed, it is very skewed. This suggests to us that some transformation of the variable may be necessary. 
// One of the commonly used transformations is log transformation. Let's try it here.

generate lggnp=log(gnpcap)
label variable lggnp "log-10 of gnpcap"
kdensity lggnp, normal

// The transformation does seem to help correct the skewness greatly. Next, let's do the regression again replacing gnpcap by lggnp.

regress birth lggnp urban

acprplot lggnp, lowess

// The plot above shows less deviation from nonlinearity than before, though the problem of nonlinearity has not been completely solved yet.

*** Potential solution [from Kohler, U. and Kreuter, F. (2005). Data Analysis Using Stata. Stata Press, Ch. 8]

* In the presence of nonlinear relations, you need to transform the independent variabless involved or include more quadrtic terms in the equation.

********************************************************************************
********************************************************************************
**************************** Model Specification *******************************
********************************************************************************

// A model specification error can occur when one or more relevant variables are omitted from the model (in this case we have Violation of E(ei)=0)
// or one or more irrelevant variables are included in the model. 
// If relevant variables are omitted from the model, the common variance they share with included variables 
// may be wrongly attributed to those variables, and the error term is inflated. 
// On the other hand, if irrelevant variables are included in the model, the common variance they share with included variables 
// may be wrongly attributed to them. Model specification errors can substantially affect the estimate of regression coefficients.

* Consider the model below

use elemapi2.dta, clear

regress api00 acs_k3

* This regression suggests that as class size increases the academic performance increases. 
// Before we publish results saying that increased class size is associated with higher academic performance, let's check the model specification.

// There are a couple of methods to detect specification errors. 
// The "linktest" (linktest - tests for specification error) command performs a model specification link test for single-equation models. 
// "linktest" is based on the idea that if a regression is properly specified, one should not be able to find any additional independent variables 
// that are significant except by chance. 
// linktest creates two new variables, the variable of prediction, _hat, and the variable of squared prediction, _hatsq. 
// The model is then refit using these two variables as predictors. _hat should be significant since it is the predicted value. 
// On the other hand, _hatsq shouldn't, because if our model is specified correctly, the squared predictions should not have much explanatory power. 
// That is we wouldn't expect _hatsq to be a significant predictor if our model is specified correctly. So we will be looking at the p-value for _hatsq.

linktest

// From the above "linktest", the test of _hatsq is not significant. 
// This is to say that "linktest" has failed to reject the assumption that the model is specified correctly. 
// Therefore, it seems to us that we don't have a specification error. But now, let's look at another test before we jump to the conclusion.

// Although linktest is formally a test of the specification of the dependent variable, 
// it is often interpreted as a test that, conditional on the specification, 
// the independent variables are specified incorrectly.

* The "ovtest" (ovtest - tests for omitted variables) command performs another test of regression model specification. 
// It performs a regression specification error test (RESET) for omitted variables. The idea behind ovtest is very similar to linktest. 
// It also creates new variables based on the predictors and refits the model using those new variables to see if any of them would be significant.  
// Let's try "ovtest" on our model.

ovtest

// The "ovtest" command indicates that there are omitted variables. So we have tried both the linktest and ovtest, 
// and one of them (ovtest) tells us that we have a specification error. We therefore have to reconsider our model. 

// option "rhs" calls for using powers of the right-hand-side variables, instead of powers of predicted y (default). 
// Criteria for selecting between the two versions of the test is unclear. 
// It is suggested that it indicates whether the specification error is likely to be related to the functional form 
// of the variables currently in the model (powers of the independent variables version), 
// or simply exogenous to the current model entirely (powers of fitted values version).

// Let's try adding the variable full to the model. Now, both the linktest and ovtest are significant, indicating we have a specification error.

regress api00 acs_k3 full

linktest

ovtest

// Let's try adding one more variable, meals, to the above model. 

regress api00 acs_k3 full meals

linktest

ovtest

// The linktest is once again non-significant while the p-value for ovtest is slightly greater than .05. 
// Note that after including meals and full, the coefficient for class size is no longer significant. 
// While acs_k3 does have a positive relationship with api00 when no other variables are in the model, 
// when we include, and hence control for, other important variables, acs_k3 is no longer significantly related to api00 
// and its relationship to api00 is no longer positive. 

* "linktest" and "ovtest" are tools available in Stata for checking specification errors, 
// though "linktest" can actually do more than check omitted variables as we used here, 
// e.g., checking the correctness of link function specification. For more details on those tests, please refer to Stata manual.

*** // The significant -linktest- suggests that a linear model may not be the best fit.
// The significant -ovtest- suggests there's a pattern in the residuals.


